//-----------------------------------------------------
//	`֘ACu
//	2010.03.04 TMO
//-----------------------------------------------------

#pragma once
#include <d3d9.h>
#include <d3d9types.h>
#include <d3dx9tex.h>

#pragma warning(disable:4482)

struct FPOINT
{
	float x;
	float y;

	FPOINT()
	{
	}
	FPOINT( float fx, float fy )
	{
		x = fx;
		y = fy;
	}
	
	FPOINT& operator + (const FPOINT& src ) {
		x += src.x;
		y += src.y;
		return *this;
	}

};

struct FRECT
{
	float left;
	float top;
	float right;
	float bottom;

	FRECT(){ left = top = right = bottom = 0; }
	FRECT(float l, float t, float r, float b)
	{
		left = l;
		top = t;
		right = r;
		bottom = b;
	}
	float Wid(){return ( right - left );}
	float Hgt(){return ( bottom - top );}

	void SetWid(float w) {
		
		left = left + ( ( right - left ) - w ) * 0.5f;
		
		right = left+w;
	}
	void SetHgt(float h) {
		top = top + ( ( bottom - top ) - h ) * 0.5f;
		bottom = top+h;
	}

	void SetCenter( const FPOINT& fp ) {
		//TOFO
		float w = Wid();
		float h = Hgt();
		left   = fp.x - ( w / 2 );
		right  = left + w;
		top    = fp.y - ( h / 2 );
		bottom = top +h;
	}
	void SetCenter( float x, float y ) {
		//TOFO
		float w = Wid();
		float h = Hgt();
		left   = x - ( w / 2 );
		right  = left + w;
		top    = y - ( h / 2 );
		bottom = top +h;
	}
	
	void Move( float x, float y ) {
		left   += x;
		top    += y;
		right  += x;
		bottom += y;
	}

	FPOINT Center()
	{
		return FPOINT( left + ( ( right - left ) * 0.5f ) , top + ( ( bottom - top )*0.5f ) );
	}
};
struct IRECT
{
	int left;
	int top;
	int right;
	int bottom;

	IRECT(){ left = top = right = bottom = 0; }
	IRECT(int l, int t, int r, int b)
	{
		left = l;
		top = t;
		right = r;
		bottom = b;
	}
	int Wid() {return right - left;}
	int Hgt() {return bottom - top;}
};

//Xt@CNX
class XFileObject
{
	DWORD		m_dwNumMaterials;

	LPDIRECT3DTEXTURE9*	m_ppTextures;
	D3DMATERIAL9*		m_pMatrs;

public:
	LPD3DXMESH	m_pMesh;

	XFileObject();
	bool Load( LPCWSTR file );
	void Render();
	void Render(D3DMATRIX& mat);

	void Release();
};

//eNX`NX
class CTexture {
	LPDIRECT3DTEXTURE9	m_texture;
public:
	CTexture();
	~CTexture();
	
	bool Load( LPCTSTR path );
	void SetCurrentUse();
	void Release();
};

//J
class Camera
{
	D3DXVECTOR3 m_pos;
	D3DXVECTOR3 m_at;
	D3DXVECTOR3 m_top;
	
	D3DXMATRIX m_mat;

public:
	void Select(
			float x, float y, float z,
			float at_x, float at_y, float at_z,
			float top_x, float top_y, float top_z );

	void Rotate( float yaw, float pitch, float roll );
	void RotateAtBase( float roty, float rotz );

	void Move( float x, float y, float z );
};

//uD3DFVF_TEX1v1̓eNX`̖
#define D3DFVF_2DVERTEX (D3DFVF_XYZRHW | D3DFVF_TEX1 | D3DFVF_DIFFUSE )
typedef struct _D3DVERTEX_
{   D3DVECTOR	pos;		//_̈ʒu
	float		rhw;		//1Ƃ
	D3DCOLOR    color;		//diffuse	|S̐FBʂ̕/S̃tF[hƂeNX`摜̃At@l𐧌䂷̂Ɏg
	FLOAT       tu;			//eNX`̍W
	FLOAT       tv;			//eNX`̍W

} D3DVERTEX;

//`֘ANX
class CGraphic
{
	enum DisplayMode {
		FullScreen,
		WindowMode,
	};

	DisplayMode	m_currDisplayMode;//ʃ[h

	int	m_screenWid;
	int	m_screenHgt;
	int m_softwareMode;
	int m_culling;

	bool LoadSetting();

	int m_RefreshRate;

	IDirect3DVertexBuffer9* m_pVB; //Draw.cppŎg_obt@

public :
	DWORD		m_backColor;//wiF
	

	CGraphic();
	~CGraphic();
	
	DisplayMode GetDisplayMode(){ return m_currDisplayMode; }
	int GetRefreshRate(){return m_RefreshRate;}

	//Ɣj
	bool Create(HWND hWnd);
	void Destroy();

	void ClearAndBegin();//DrawOɌĂ
	void EndAndPresent();//DrawɌĂ

	void DrawColorRect( float left, float top, float right, float bottom, DWORD color );
	void Draw( float left, float top, float right, float bottom, DWORD alphaCol );
	void Draw( float left, float top, float right, float bottom, CTexture* texture, DWORD alphaCol );


	void SetZEnable(BOOL isOn);
	void SetBlendingDefault();
	void SetBlendingAdd();

};

//eLXg֘A
#include "Graphic_Text.h"


//CGraphicNX̐Ɣj͂ōsB
CGraphic*	CreateGraphic(HWND hWnd);
CGraphic*	GetGraphic();
void		ReleaseGraphic();